from zona.models import Item, Metadata
from pathlib import Path
import frontmatter
from dacite import from_dict


def split_metadata(path: Path) -> tuple[Metadata, str]:
    post = frontmatter.load(str(path))
    # TODO: handle missing metadata
    meta = post.metadata
    # TODO: handle malformed metadata
    metadata = from_dict(
        data_class=Metadata,
        data=meta,
    )
    return metadata, post.content


def discover(root: Path, out_dir: Path) -> list[Item]:
    required_dirs = {"content", "templates"}
    found_dirs = {p.name for p in root.iterdir() if p.is_dir()}
    missing = required_dirs - found_dirs
    assert not missing, f"Missing required directories: {', '.join(missing)}"

    items: list[Item] = []

    subdir = "content"
    base = root / subdir
    for path in base.rglob("*"):
        if path.is_file():
            print(f"{subdir}: {path.relative_to(root)}")
            if path.name.endswith(".md") and not path.is_relative_to(
                root / "content" / "static"
            ):
                meta, content = split_metadata(path)
            else:
                meta = None
                content = path.read_bytes()
            destination = out_dir / path.relative_to(base)
            item = Item(
                source=path,
                destination=destination,
                url=str(destination.relative_to(out_dir)),
                metadata=meta,
                content=content,
            )
            items.append(item)
    return items
